@@ -2,15 +2,15 @@ from .client import *
 from .feature_derivations import *
 from .anchor import *
 from .feature import *
-from .source import *
 from .dtype import *
 from .source import *
 from .transformation import *
 from .transformation import *
 from .typed_key import *
 from .materialization_settings import (BackfillTime, MaterializationSettings)
-from .sink import RedisSink
+from .sink import RedisSink, HdfsSink
 from .query_feature_list import FeatureQuery
 from .lookup_feature import LookupFeature
-from .aggregation import  Aggregation
-from .feathr_configurations import SparkExecutionConfiguration
\ No newline at end of file
+from .aggregation import Aggregation
+from .feathr_configurations import SparkExecutionConfiguration
+from .api.app.core.feathr_api_exception import *
\ No newline at end of file
