@@ -1,11 +1,13 @@
 import os
 import yaml
 from loguru import logger
+from feathr.akv_client import AzureKeyVaultClient
 
 
 class _EnvVaraibleUtil(object):
     def __init__(self, config_path):
         self.config_path = config_path
+        self.akv_client = None
 
     def get_environment_variable_with_default(self, *args):
         """Gets the environment variable for the variable key.
@@ -29,7 +31,7 @@ class _EnvVaraibleUtil(object):
         try:
             assert os.path.exists(os.path.abspath(self.config_path))
         except:
-            logger.info("{} is not set and configuration file {} cannot be found. One of those shoudl be set." , env_keyword, self.config_path)
+            logger.info("{} is not set and configuration file {} cannot be found. One of those should be set." , env_keyword, self.config_path)
 
         with open(os.path.abspath(self.config_path), 'r') as stream:
             try:
@@ -42,6 +44,9 @@ class _EnvVaraibleUtil(object):
                 for arg in args:
                     yaml_layer = yaml_layer[arg]
                 return yaml_layer
+            except KeyError as exc:
+                logger.info(exc)
+                return ""
             except yaml.YAMLError as exc:
                 logger.info(exc)
 
@@ -59,5 +64,10 @@ class _EnvVaraibleUtil(object):
         password = os.environ.get(variable_key)
         if not password:
             logger.info(variable_key +
-                        ' is not set in the environment variables.')
+                        ' is not set in the environment variables, fetching the value from Key Vault')
+            akv_name = os.environ.get("KEY_VAULT_NAME")
+            if akv_name:
+                akv_client = AzureKeyVaultClient(akv_name)
+                password = akv_client.get_akv_secret(variable_key)
         return password
+
