@@ -1,8 +1,11 @@
 import glob
 import importlib
 import inspect
+import itertools
 import os
 import sys
+import ast
+import types
 from graphlib import TopologicalSorter
 from pathlib import Path
 from tracemalloc import stop
@@ -20,14 +23,15 @@ from pyapacheatlas.core import (AtlasClassification, AtlasEntity, AtlasProcess,
 from pyapacheatlas.core.typedef import (AtlasAttributeDef,
                                         AtlasRelationshipEndDef, Cardinality,
                                         EntityTypeDef, RelationshipTypeDef)
+                            
 from pyapacheatlas.core.util import GuidTracker
 from pyhocon import ConfigFactory
 
-from feathr._envvariableutil import _EnvVaraibleUtil
 from feathr._file_utils import write_to_file
 from feathr.anchor import FeatureAnchor
 from feathr.constants import *
-from feathr.feature import Feature, FeatureType
+from feathr.dtype import *
+from feathr.feature import Feature, FeatureBase, FeatureType
 from feathr.feature_derivations import DerivedFeature
 from feathr.repo_definitions import RepoDefinitions
 from feathr.source import HdfsSource, InputContext, Source
@@ -52,9 +56,9 @@ class _FeatureRegistry():
         self.credential = DefaultAzureCredential(exclude_interactive_browser_credential=False) if credential is None else credential
         self.oauth = AzCredentialWrapper(credential=self.credential)
         self.purview_client = PurviewClient(
-            account_name=self.azure_purview_name,
-            authentication=self.oauth
-        )
+                account_name=self.azure_purview_name,
+                authentication=self.oauth
+            )   
         self.guid = GuidTracker(starting=-1000)
         self.entity_batch_queue = []
 
@@ -214,13 +218,16 @@ class _FeatureRegistry():
             anchor_feature_entities = self._parse_anchor_features(anchor)
             # then parse the source of that anchor
             source_entity = self._parse_source(anchor.source)
-
+            anchor_fully_qualified_name  = self.project_name+self.registry_delimiter+anchor.name
+            original_id = self.get_feature_id(anchor_fully_qualified_name )
+            original_anchor = self.get_feature_by_guid(original_id) if original_id else None
+            merged_elements = self._merge_anchor(original_anchor,anchor_feature_entities)
             anchor_entity = AtlasEntity(
                 name=anchor.name,
-                qualified_name=self.project_name + self.registry_delimiter + anchor.name,
+                qualified_name=anchor_fully_qualified_name ,
                 attributes={
                     "source": source_entity.to_json(minimum=True),
-                    "features": [s.to_json(minimum=True) for s in anchor_feature_entities],
+                    "features": merged_elements,
                     "tags": anchor.registry_tags
                 },
                 typeName=TYPEDEF_ANCHOR,
@@ -256,6 +263,30 @@ class _FeatureRegistry():
             anchors_batch.append(anchor_entity)
         return anchors_batch
 
+    def _merge_anchor(self,original_anchor:Dict, new_anchor:Dict)->List[Dict[str,any]]:
+        '''
+        Merge the new anchors defined locally with the anchors that is defined in the centralized registry.
+        '''
+        # TODO: This will serve as a quick fix, full fix will work with MVCC, and is in progress.
+        new_anchor_json_repr = [s.to_json(minimum=True) for s in new_anchor]
+        if not original_anchor:
+            # if the anchor is not present on the registry, return json representation of the locally defined anchor.
+            # sample : [{'guid':'GUID_OF_ANCHOR','typeName':'','qualifiedName':'QUALIFIED_NAME'}
+            return new_anchor_json_repr
+        else:
+            original_anchor_elements = [x for x in original_anchor['entity']['attributes']['features']]
+            transformed_original_elements = {
+                x['uniqueAttributes']['qualifiedName']:
+                {
+                    'guid':x['guid'],
+                    'typeName':x['typeName'],
+                    'qualifiedName':x['uniqueAttributes']['qualifiedName']
+                }
+                for x in original_anchor_elements}
+            for elem in new_anchor_json_repr:
+                transformed_original_elements.setdefault(elem['qualifiedName'],elem)
+            return list(transformed_original_elements.values())
+            
     def _parse_source(self, source: Union[Source, HdfsSource]) -> AtlasEntity:
         """
         parse the input sources
@@ -304,12 +335,20 @@ class _FeatureRegistry():
         for derived_feature in derived_features:
             # make sure the input is derived feature
             if isinstance(derived_feature, DerivedFeature):
+                # add this derived feature in the topo sort graph without any precessesors
+                # since regardless we need it
+                ts.add(derived_feature)
                 for input_feature in derived_feature.input_features:
                     if isinstance(input_feature, DerivedFeature):
                         # `input_feature` is predecessor of `derived_feature`
                         ts.add(derived_feature, input_feature)
+                        # if any of the input feature is a derived feature, have this recursive call
+                        # use this for code simplicity. 
+                        # if the amount of features is huge, consider only add the derived features into the function call
                         self._add_all_derived_features(input_feature.input_features, ts)
 
+                        
+
 
     def _parse_derived_features(self, derived_features: List[DerivedFeature]) -> List[AtlasEntity]:
         """parse derived feature
@@ -362,7 +401,8 @@ class _FeatureRegistry():
                     "input_anchor_features": [f.to_json(minimum=True) for f in input_feature_entity_list if f.typeName==TYPEDEF_ANCHOR_FEATURE],
                     "input_derived_features": [f.to_json(minimum=True) for f in input_feature_entity_list if f.typeName==TYPEDEF_DERIVED_FEATURE],
                     "transformation": transform_dict,
-                    "tags": derived_feature.registry_tags
+                    "tags": derived_feature.registry_tags,
+
                 },
                 typeName=TYPEDEF_DERIVED_FEATURE,
                 guid=self.guid.get_guid(),
@@ -657,11 +697,12 @@ derivations: {
         logger.info(
             "Finished registering features. See {} to access the Purview web interface", webinterface_path)
 
+
     def _purge_feathr_registry(self):
         """
         Delete all the feathr related entities and type definitions in feathr registry. For internal use only
         """
-        self._delete_all_feathr_entitties()
+        self._delete_all_feathr_entities()
         self._delete_all_feathr_types()
 
 
@@ -686,7 +727,7 @@ derivations: {
 
         logger.info("Deleted all the Feathr related definitions.")
 
-    def _delete_all_feathr_entitties(self):
+    def _delete_all_feathr_entities(self):
         """
         Delete all the corresonding entity for feathr registry. For internal use only
 
@@ -704,14 +745,15 @@ derivations: {
             self.purview_client.delete_entity(
                 guid=guid_list[i:i+batch_delte_size])
             logger.info("{} feathr entities deleted", batch_delte_size)
-
+    
+    @classmethod
     def _get_registry_client(self):
         """
         Return a client object and users can operate more on it (like doing search)
         """
         return self.purview_client
 
-    def list_registered_features(self, project_name: str = None, limit=50, starting_offset=0) -> List[str]:
+    def list_registered_features(self, project_name: str = None, limit=50, starting_offset=0) -> List[Dict[str,str]]:
         """
         List all the already registered features. If project_name is not provided or is None, it will return all the
         registered features; otherwise it will only return only features under this project
@@ -727,18 +769,71 @@ derivations: {
                 # split the name based on delimiter
                 result = qualified_name.split(self.registry_delimiter)
                 if result[0].casefold() == project_name:
-                    feature_list.append(entity["name"])
+                    feature_list.append({"name":entity["name"],'id':entity['id'],"qualifiedName":entity['qualifiedName']})
             else:
                 # otherwise append all the entities
-                feature_list.append(entity["name"])
+                feature_list.append({"name":entity["name"],'id':entity['id'],"qualifiedName":entity['qualifiedName']})
 
         return feature_list
+   
+    def get_feature_by_fqdn_type(self, qualifiedName, typeName):
+        """
+        Get a single feature by it's QualifiedName and Type
+        Returns the feature else throws an AtlasException with 400 error code
+        """
+        response = self.purview_client.get_entity(qualifiedName=qualifiedName, typeName=typeName)
+        entities = response.get('entities')
+        for entity in entities:
+            if entity.get('typeName') == typeName and entity.get('attributes').get('qualifiedName') == qualifiedName: 
+                return entity
+       
+    def get_feature_by_fqdn(self, qualifiedName):
+        """
+        Get feature by qualifiedName
+        Returns the feature else throws an AtlasException with 400 error code
+        """        
+        id = self.get_feature_id(qualifiedName)
+        return self.get_feature_by_guid(id)
+    
+    def get_feature_by_guid(self, guid):
+        """
+        Get a single feature by it's GUID
+        Returns the feature else throws an AtlasException with 400 error code
+        """ 
+        response = self.purview_client.get_single_entity(guid=guid)
+        return response
+    
+    def get_feature_lineage(self, guid):
+        """
+        Get feature's lineage by it's GUID
+        Returns the feature else throws an AtlasException with 400 error code
+        """
+        return self.purview_client.get_entity_lineage(guid=guid)
 
+    def get_feature_id(self, qualifiedName):
+        """
+        Get guid of a feature given its qualifiedName
+        """        
+        search_term = "qualifiedName:{0}".format(qualifiedName)
+        entities = self.purview_client.discovery.search_entities(search_term)
+        for entity in entities:
+            if entity.get('qualifiedName') == qualifiedName:
+                return entity.get('id')
+
+    def search_features(self, searchTerm):
+        """
+        Search the registry for the given query term
+        For a ride hailing company few examples could be - "taxi", "passenger", "fare" etc.
+        It's a keyword search on the registry metadata
+        """        
+        entities = self.purview_client.discovery.search_entities(searchTerm)
+        return entities
+    
     def _list_registered_entities_with_details(self, project_name: str = None, entity_type: Union[str, List[str]] = None, limit=50, starting_offset=0,) -> List[Dict]:
         """
         List all the already registered entities. entity_type should be one of: SOURCE, DERIVED_FEATURE, ANCHOR, ANCHOR_FEATURE, FEATHR_PROJECT, or a list of those values
         limit: a maximum 1000 will be enforced at the underlying API
-
+        
         returns a list of the result entities.
         """
         entity_type_list = [entity_type] if isinstance(
@@ -749,7 +844,8 @@ derivations: {
                 raise RuntimeError(
                     f'only SOURCE, DERIVED_FEATURE, ANCHOR, ANCHOR_FEATURE, FEATHR_PROJECT are supported when listing the registered entities, {entity_type} is not one of them.')
 
-        # the search grammar is less documented in Atlas/Purview
+        # the search grammar is less documented in Atlas/Purview. 
+        # Here's the query grammar: https://atlas.apache.org/2.0.0/Search-Advanced.html
         search_string = "".join(
             [f" or entityType:{e}" for e in entity_type_list])
         # remvoe the first additional " or "
@@ -772,6 +868,196 @@ derivations: {
             else:
                 # otherwise append all the entities
                 guid_list.append(entity["id"])
-        entity_res = [] if guid_list is None else self.purview_client.get_entity(
+        entity_res = [] if guid_list is None or len(guid_list)==0 else self.purview_client.get_entity(
             guid=guid_list)["entities"]
         return entity_res
+        
+    def get_features_from_registry(self, project_name: str) -> Tuple[List[FeatureAnchor], List[DerivedFeature]]:
+        """Sync Features from registry to local workspace, given a project_name, will write project's features from registry to to user's local workspace]
+        If the project is big, the return result could be huge.
+        Args:
+            project_name (str): project name.
+        """
+
+        entities = self._list_registered_entities_with_details(project_name=project_name,entity_type=[TYPEDEF_DERIVED_FEATURE, TYPEDEF_ANCHOR_FEATURE, TYPEDEF_FEATHR_PROJECT])
+        if not entities:
+            # if the result is empty
+            return (None, None)
+        
+        # get project entity, the else are feature entities (derived+anchor)
+        project_entity = [x for x in entities if x['typeName']==TYPEDEF_FEATHR_PROJECT][0] # there's only one available
+        feature_entities = [x for x in entities if x!=project_entity]
+        feature_entity_guid_mapping = {x['guid']:x for x in feature_entities}
+
+        # this is guid for feature anchor (GROUP of anchor features)
+        anchor_guid = [anchor_entity["guid"] for anchor_entity in project_entity["attributes"]["anchor_features"]]
+        derived_feature_guid = [derived_feature_entity["guid"] for derived_feature_entity in project_entity["attributes"]["derived_features"]]
+        
+        derived_feature_ids = [feature_entity_guid_mapping[x] for x in derived_feature_guid]
+        
+        derived_feature_list = []
+        for derived_feature_entity_id in derived_feature_ids:
+            # this will be used to generate DerivedFeature instance
+            derived_feature_key_list = []
+
+                
+            for key in derived_feature_entity_id["attributes"]["key"]:
+                derived_feature_key_list.append(TypedKey(key_column=key["key_column"], key_column_type=key["key_column_type"], full_name=key["full_name"], description=key["description"], key_column_alias=key["key_column_alias"]))
+            
+            # for feature anchor (GROUP), input features are splitted into input anchor features & input derived features
+            anchor_feature_guid = [e["guid"] for e in derived_feature_entity_id["attributes"]["input_anchor_features"]]
+            derived_feature_guid = [e["guid"] for e in derived_feature_entity_id["attributes"]["input_derived_features"]]
+            
+            # for derived features, search all related input features.
+            input_features_guid = self.search_input_anchor_features(derived_feature_guid,feature_entity_guid_mapping)
+
+            # chain the input features together
+            all_input_features = list(itertools.chain.from_iterable(
+                [self._get_features_by_guid(x) for x in input_features_guid+anchor_feature_guid]))
+
+            derived_feature_list.append(DerivedFeature(name=derived_feature_entity_id["attributes"]["name"],
+                                feature_type=self._get_feature_type_from_hocon(derived_feature_entity_id["attributes"]["type"]),
+                                transform=self._get_transformation_from_dict(derived_feature_entity_id["attributes"]['transformation']),
+                                key=derived_feature_key_list,
+                                input_features= all_input_features,
+                                registry_tags=derived_feature_entity_id["attributes"]["tags"]))                    
+        anchor_result = self.purview_client.get_entity(guid=anchor_guid)["entities"]
+        anchor_list = []
+        for anchor_entity in anchor_result:
+            feature_guid = [e["guid"] for e in anchor_entity["attributes"]["features"]]
+            anchor_list.append(FeatureAnchor(name=anchor_entity["attributes"]["name"],
+                                source=self._get_source_by_guid(anchor_entity["attributes"]["source"]["guid"]),
+                                features=self._get_features_by_guid(feature_guid),
+                                registry_tags=anchor_entity["attributes"]["tags"]))
+
+        
+        return (anchor_list, derived_feature_list)
+
+    def search_input_anchor_features(self,derived_guids,feature_entity_guid_mapping) ->List[str]:
+        '''
+        Iterate all derived features and its parent links, extract and aggregate all inputs
+        '''
+        stack = [x for x in derived_guids]
+        result = []
+        while len(stack)>0:
+            current_derived_guid = stack.pop()
+            current_input = feature_entity_guid_mapping[current_derived_guid]
+            new_derived_features = [x["guid"] for x in current_input["attributes"]["input_derived_features"]]
+            new_anchor_features = [x["guid"] for x in current_input["attributes"]["input_anchor_features"]]
+            for feature_guid in new_derived_features:
+                stack.append(feature_guid)
+            result += new_anchor_features
+            result = list(set(result))
+        return result
+
+
+    def _correct_function_identation(self, user_func: str) -> str:
+        """
+        The function read from registry might have the wrong identation. We need to correct those identations.
+        More specifically, we are using the inspect module to copy the function body for UDF for further submission. In that case, there will be situations like this:
+
+        def feathr_udf1(df)
+            return df
+
+                def feathr_udf2(df)
+                    return df
+
+        For example, in Feathr test cases, there are similar patterns for `feathr_udf2`, since it's defined in another function body (the registry_test_setup method).
+        This is not an ideal way of dealing with that, but we'll keep it here until we figure out a better way.
+        """
+        if user_func is None:
+            return None
+        # if user_func is a string, turn it into a list of strings so that it can be used below
+        temp_udf_source_code = user_func.split('\n')
+        # assuming the first line is the function name
+        leading_space_num = len(temp_udf_source_code[0]) - len(temp_udf_source_code[0].lstrip())
+        # strip the lines to make sure the function has the correct indentation
+        udf_source_code_striped = [line[leading_space_num:] for line in temp_udf_source_code]
+        # append '\n' back since it was deleted due to the previous split
+        udf_source_code = [line+'\n' for line in udf_source_code_striped]
+        return " ".join(udf_source_code)
+
+    def _get_source_by_guid(self, guid) -> Source:
+        # TODO: currently return HDFS source by default. For JDBC source, it's currently implemented using HDFS Source so we should split in the future
+        source_entity = self.purview_client.get_entity(guid=guid)["entities"][0]
+
+        # if source_entity["attributes"]["path"] is INPUT_CONTEXT, it will also be assigned to this returned object
+        return HdfsSource(name=source_entity["attributes"]["name"],
+                event_timestamp_column=source_entity["attributes"]["event_timestamp_column"],
+                timestamp_format=source_entity["attributes"]["timestamp_format"],
+                preprocessing=self._correct_function_identation(source_entity["attributes"]["preprocessing"]),
+                path=source_entity["attributes"]["path"],
+                registry_tags=source_entity["attributes"]["tags"]
+                )
+
+
+
+    def _get_feature_type_from_hocon(self, input_str: str) -> FeatureType:
+        """Get Feature types from a HOCON config, given that we stored the feature type in a plain string.
+
+        Args:
+            input_str (str): the input string for the stored HOCON config
+
+        Returns:
+            FeatureType: feature type that can be used by Python
+        """
+        conf = ConfigFactory.parse_string(input_str)
+        valType = conf.get_string('type.valType')
+        dimensionType = conf.get_string('type.dimensionType')
+        if dimensionType == '[INT]':
+            # if it's not empty, i.e. [INT], indicating it's vectors
+            if valType == 'DOUBLE':
+                return DoubleVectorFeatureType()
+            elif valType == 'LONG':
+                return Int64VectorFeatureType()
+            elif valType == 'INT':
+                return Int32VectorFeatureType()
+            elif valType == 'FLOAT':
+                return FloatVectorFeatureType()
+            else:
+                logger.error("{} cannot be parsed.", valType)
+        else:
+            if valType == 'STRING':
+                return StringFeatureType()
+            elif valType == 'BYTES':
+                return BytesFeatureType()
+            elif valType == 'DOUBLE':
+                return DoubleFeatureType()
+            elif valType == 'FLOAT':
+                return FloatFeatureType()
+            elif valType == 'LONG':
+                return Int64FeatureType()
+            elif valType == 'INT':
+                return Int32FeatureType()
+            elif valType == 'BOOLEAN':
+                return BooleanFeatureType()
+            else:
+                logger.error("{} cannot be parsed.", valType)
+
+    def _get_transformation_from_dict(self, input: Dict) -> FeatureType:
+        if 'transform_expr' in input:
+            # it's ExpressionTransformation
+            return ExpressionTransformation(input['transform_expr'])
+        elif 'def_expr' in input:
+            return WindowAggTransformation(agg_expr=input['def_expr'], agg_func=input['agg_func'], window=input['window'], group_by=input['group_by'], filter=input['filter'], limit=input['limit'])
+        else:
+            # no transformation function observed
+            return None
+
+    def _get_features_by_guid(self, guid) -> List[FeatureAnchor]:
+        feature_entities = self.purview_client.get_entity(guid=guid)["entities"]
+        feature_list=[]
+        key_list = []
+        for feature_entity in feature_entities:
+            for key in feature_entity["attributes"]["key"]:
+                key_list.append(TypedKey(key_column=key["key_column"], key_column_type=key["key_column_type"], full_name=key["full_name"], description=key["description"], key_column_alias=key["key_column_alias"]))
+
+            # after get keys, put them in features
+            feature_list.append(Feature(name=feature_entity["attributes"]["name"],
+                    feature_type=self._get_feature_type_from_hocon(feature_entity["attributes"]["type"]), # stored as a hocon string, can be parsed using pyhocon
+                    transform=self._get_transformation_from_dict(feature_entity["attributes"]['transformation']), #transform attributes are stored in a dict fashion , can be put in a WindowAggTransformation
+                    key=key_list,
+                    registry_tags=feature_entity["attributes"]["tags"],
+
+            ))
+        return feature_list 
