@@ -4,19 +4,22 @@ from feathr.source import Source
 from feathr.typed_key import DUMMY_KEY
 from jinja2 import Template
 from feathr.source import INPUT_CONTEXT
+from feathr.frameconfig import HoconConvertible
 
-# passthrough features do not need keys
-class FeatureAnchor:
+
+class FeatureAnchor(HoconConvertible):
     """
-    A feature anchor defines a set of features on top of a data source, a.k.a. a set of features anchored to a source.
+        A feature anchor defines a set of features on top of a data source, a.k.a. a set of features anchored to a source.
 
-    The feature producer writes multiple anchors for a feature, exposing the same feature name for the feature
-    consumer to reference it.
-    Attributes:
-        name: Unique name of the anchor.
-        source: data source that the features are anchored to. Should be either of `INPUT_CONTEXT` or `feathr.source.Source`
-        features: list of features within this anchor. 
-        registry_tags: A dict of (str, str) that you can pass to feature registry for better organization. For example, you can use {"deprecated": "true"} to indicate this anchor is deprecated, etc.
+        The feature producer writes multiple anchors for a feature, exposing the same feature name for the feature
+        consumer to reference it.
+
+        Attributes:
+            name: Unique name of the anchor.
+            source: data source that the features are anchored to. Should be either of `INPUT_CONTEXT` or `feathr.source.Source`
+            features: list of features defined within this anchor.
+            registry_tags: A dict of (str, str) that you can pass to feature registry for better organization.
+                           For example, you can use {"deprecated": "true"} to indicate this anchor is deprecated, etc.
     """
     def __init__(self,
                 name: str,
@@ -38,9 +41,7 @@ class FeatureAnchor:
                 if feature.key == [DUMMY_KEY]:
                     raise RuntimeError(f"For anchors of non-INPUT_CONTEXT source, key of feature {feature.name} "
                                        f"should be explicitly specified and not left blank.")
-        for feature in self.features:
-            assert feature.key_alias == self.features[0].key_alias
-
+        
     def to_feature_config(self) -> str:
         tm = Template("""
             {{anchor_name}}: {
