@@ -4,7 +4,9 @@ import os
 import tempfile
 from datetime import datetime, timedelta
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Union
+from feathr.utils import FeaturePrinter
+from feathr.feature import FeatureBase
 
 import redis
 from azure.identity import DefaultAzureCredential
@@ -21,12 +23,11 @@ from feathr._synapse_submission import _FeathrSynapseJobLauncher
 from feathr.constants import *
 from feathr.feathr_configurations import SparkExecutionConfiguration
 from feathr.feature_derivations import DerivedFeature
+from feathr.anchor import FeatureAnchor
 from feathr.materialization_settings import MaterializationSettings
 from feathr.protobuf.featureValue_pb2 import FeatureValue
 from feathr.query_feature_list import FeatureQuery
 from feathr.settings import ObservationSettings
-from feathr.feature_derivations import DerivedFeature
-from feathr.anchor import FeatureAnchor
 from feathr.feathr_configurations import SparkExecutionConfiguration
 
 
@@ -73,7 +74,7 @@ class FeathrClient(object):
     The users of this client is responsible for set up all the necessary information needed to start a Redis client via
     environment variable or a Spark cluster. Host address, port and password are needed to start the Redis client.
 
-    Args:
+    Attributes:
         config_path (str, optional): config path. See [Feathr Config Template](https://github.com/linkedin/feathr/blob/main/feathr_project/feathrcli/data/feathr_user_workspace/feathr_config.yaml) for more details.  Defaults to "./feathr_config.yaml".
         local_workspace_dir (str, optional): set where is the local work space dir. If not set, Feathr will create a temporary folder to store local workspace related files.
         credential (optional): credential to access cloud resources,  most likely to be the returned result of DefaultAzureCredential(). If not set, Feathr will initialize DefaultAzureCredential() inside the __init__ function to get credentials.
@@ -91,7 +92,9 @@ class FeathrClient(object):
         if local_workspace_dir:
             self.local_workspace_dir = local_workspace_dir
         else:
-            self.local_workspace_dir = tempfile.TemporaryDirectory().name
+            # this is required for Windows
+            tem_dir_obj = tempfile.TemporaryDirectory()
+            self.local_workspace_dir = tem_dir_obj.name
 
         self.envutils = envutils
 
@@ -132,7 +135,7 @@ class FeathrClient(object):
             self._FEATHR_JOB_JAR_PATH = \
                 envutils.get_environment_variable_with_default(
                     'spark_config', 'azure_synapse', 'feathr_runtime_location')
-            
+
             if self.credential is None:
                 self.credential = DefaultAzureCredential(exclude_interactive_browser_credential=False)
 
@@ -207,9 +210,10 @@ class FeathrClient(object):
                 raise RuntimeError("Please call FeathrClient.build_features() first in order to register features")
         else:
             self.registry.register_features(self.local_workspace_dir, from_context=from_context)
-    
-    def build_features(self, anchor_list: List[FeatureAnchor] = [], derived_feature_list: List[DerivedFeature] = []):
-        """Registers features based on the current workspace
+
+    def build_features(self, anchor_list: List[FeatureAnchor] = [], derived_feature_list: List[DerivedFeature] = [], verbose: bool = False):
+        """Build features based on the current workspace. all actions that triggers a spark job will be based on the
+        result of this action.
         """
         # Run necessary validations
         # anchor name and source name should be unique
@@ -232,6 +236,10 @@ class FeathrClient(object):
         self.registry.save_to_feature_config_from_context(anchor_list, derived_feature_list, self.local_workspace_dir)
         self.anchor_list = anchor_list
         self.derived_feature_list = derived_feature_list
+        
+        # Pretty print anchor_list
+        if verbose and self.anchor_list:
+                FeaturePrinter.pretty_print_anchors(self.anchor_list)
 
     def list_registered_features(self, project_name: str = None) -> List[str]:
         """List all the already registered features. If project_name is not provided or is None, it will return all
@@ -393,6 +401,7 @@ class FeathrClient(object):
                              output_path: str,
                              execution_configuratons: Union[SparkExecutionConfiguration ,Dict[str,str]] = None,
                              udf_files = None,
+                             verbose: bool = False
                              ):
         """
         Get offline features for the observation dataset
@@ -412,10 +421,10 @@ class FeathrClient(object):
 
         # produce join config
         tm = Template("""
-            {{observation_settings.to_config()}}
+            {{observation_settings.to_feature_config()}}
             featureList: [
                 {% for list in feature_lists %}
-                    {{list.to_config()}}
+                    {{list.to_feature_config()}}
                 {% endfor %}
             ]
             outputPath: "{{output_path}}"
@@ -431,6 +440,10 @@ class FeathrClient(object):
             _FeatureRegistry.save_to_feature_config_from_context(self.anchor_list, self.derived_feature_list, self.local_workspace_dir)
         else:
             raise RuntimeError("Please call FeathrClient.build_features() first in order to get offline features")
+        
+        # Pretty print feature_query
+        if verbose and feature_query:
+            FeaturePrinter.pretty_print_feature_query(feature_query)
 
         write_to_file(content=config, full_file_name=config_file_path)
         return self._get_offline_features_with_config(config_file_path, execution_configuratons, udf_files=udf_files)
@@ -509,7 +522,7 @@ class FeathrClient(object):
         else:
             raise RuntimeError('Spark job failed.')
 
-    def materialize_features(self, settings: MaterializationSettings, execution_configuratons: Union[SparkExecutionConfiguration ,Dict[str,str]] = None):
+    def materialize_features(self, settings: MaterializationSettings, execution_configuratons: Union[SparkExecutionConfiguration ,Dict[str,str]] = None, verbose: bool = False):
         """Materialize feature data
 
         Args:
@@ -517,7 +530,6 @@ class FeathrClient(object):
             execution_configuratons: a dict that will be passed to spark job when the job starts up, i.e. the "spark configurations". Note that not all of the configuration will be honored since some of the configurations are managed by the Spark platform, such as Databricks or Azure Synapse. Refer to the [spark documentation](https://spark.apache.org/docs/latest/configuration.html) for a complete list of spark configurations.
         """
         # produce materialization config
-
         for end in settings.get_backfill_cutoff_time():
             settings.backfill_time.end = end
             config = _to_materialization_config(settings)
@@ -538,6 +550,10 @@ class FeathrClient(object):
             self._materialize_features_with_config(config_file_path, execution_configuratons, udf_files)
             if os.path.exists(config_file_path):
                 os.remove(config_file_path)
+        
+        # Pretty print feature_names of materialized features
+        if verbose and settings:
+            FeaturePrinter.pretty_print_materialize_features(settings)
 
     def _materialize_features_with_config(self, feature_gen_conf_path: str = 'feature_gen_conf/feature_gen.conf',execution_configuratons: Dict[str,str] = None, udf_files=[]):
         """Materializes feature data based on the feature generation config. The feature
@@ -558,6 +574,9 @@ class FeathrClient(object):
         - Job configuration are like "configurations" for the spark job and are usually spark specific. For example, we want to control the no. of write parts for spark
         Job configurations and job arguments (or sometimes called job parameters) have quite some overlaps (i.e. you can achieve the same goal by either using the job arguments/parameters vs. job configurations). But the job tags should just be used for metadata purpose.
         '''
+        optional_params = []
+        if _EnvVaraibleUtil.get_environment_variable('KAFKA_SASL_JAAS_CONFIG'):
+            optional_params = optional_params + ['--kafka-config', self._get_kafka_config_str()]
         return self.feathr_spark_laucher.submit_feathr_job(
             job_name=self.project_name + '_feathr_feature_materialization_job',
             main_jar_path=self._FEATHR_JOB_JAR_PATH,
@@ -575,7 +594,7 @@ class FeathrClient(object):
                 '--blob-config', self._get_blob_config_str(),
                 '--sql-config', self._get_sql_config_str(),
                 '--snowflake-config', self._get_snowflake_config_str()
-            ],
+            ] + optional_params,
             reference_files_path=[],
             configuration=execution_configuratons,
         )
@@ -683,3 +702,28 @@ class FeathrClient(object):
             JDBC_SF_PASSWORD: {JDBC_SF_PASSWORD}
             """.format(JDBC_SF_URL=sf_url, JDBC_SF_USER=sf_user, JDBC_SF_PASSWORD=sf_password, JDBC_SF_ROLE=sf_role)
         return config_str
+
+    def _get_kafka_config_str(self):
+        """Construct the Kafka config string. The endpoint, access key, secret key, and other parameters can be set via
+        environment variables."""
+        sasl = _EnvVaraibleUtil.get_environment_variable('KAFKA_SASL_JAAS_CONFIG')
+        # HOCCON format will be parsed by the Feathr job
+        config_str = """
+            KAFKA_SASL_JAAS_CONFIG: "{sasl}"
+            """.format(sasl=sasl)
+        return config_str
+
+    def get_features_from_registry(self, project_name: str) -> Dict[str, FeatureBase]:
+        """
+        Get feature from registry by project name. The features got from registry are automatically built.
+        """
+        registry_anchor_list, registry_derived_feature_list = self.registry.get_features_from_registry(project_name)
+        self.build_features(registry_anchor_list, registry_derived_feature_list)
+        feature_dict = {}
+        # add those features into a dict for easier lookup
+        for anchor in registry_anchor_list:
+            for feature in anchor.features:
+                feature_dict[feature.name] = feature
+        for feature in registry_derived_feature_list:
+                feature_dict[feature.name] = feature
+        return feature_dict
