@@ -15,7 +15,7 @@ from pyspark.sql import SparkSession, DataFrame
 from test_fixture import basic_test_setup
 from feathr import (BackfillTime, MaterializationSettings)
 from feathr import RedisSink
-from test_fixture import snowflake_test_setup
+from test_fixture import (snowflake_test_setup, get_online_test_table_name)
 
 def trip_distance_preprocessing(df: DataFrame):
     df = df.withColumn("trip_distance", df.trip_distance.cast('double') - 90000)
@@ -59,7 +59,7 @@ def test_non_swa_feature_gen_with_offline_preprocessing():
     client = basic_test_setup(os.path.join(test_workspace_dir, "feathr_config.yaml"))
 
     batch_source = HdfsSource(name="nycTaxiBatchSource_add_new_fare_amount",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                               preprocessing=add_new_fare_amount,
                               event_timestamp_column="lpep_dropoff_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -87,8 +87,7 @@ def test_non_swa_feature_gen_with_offline_preprocessing():
 
     client.build_features(anchor_list=[regular_anchor])
 
-    now = datetime.now()
-    online_test_table = ''.join(['nycTaxiCITable', '_', str(now.minute), '_', str(now.second)])
+    online_test_table = get_online_test_table_name('nycTaxiCITable')
 
     backfill_time = BackfillTime(start=datetime(
         2020, 5, 20), end=datetime(2020, 5, 20), step=timedelta(days=1))
@@ -103,7 +102,7 @@ def test_non_swa_feature_gen_with_offline_preprocessing():
     client.materialize_features(settings)
     # just assume the job is successful without validating the actual result in Redis. Might need to consolidate
     # this part with the test_feathr_online_store test case
-    client.wait_job_to_finish(timeout_sec=600)
+    client.wait_job_to_finish(timeout_sec=900)
 
     res = client.get_online_features(online_test_table, '2020-04-01 07:21:51', [
         'f_is_long_trip_distance', 'f_day_of_week'])
@@ -118,7 +117,7 @@ def test_feature_swa_feature_gen_with_preprocessing():
     client = basic_test_setup(os.path.join(test_workspace_dir, "feathr_config.yaml"))
 
     batch_source = HdfsSource(name="nycTaxiBatchSource",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                               preprocessing=add_new_dropoff_and_fare_amount_column,
                               event_timestamp_column="new_lpep_dropoff_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -149,8 +148,7 @@ def test_feature_swa_feature_gen_with_preprocessing():
 
     client.build_features(anchor_list=[agg_anchor])
 
-    now = datetime.now()
-    online_test_table = ''.join(['nycTaxiCITable', '_', str(now.minute), '_', str(now.second)])
+    online_test_table = get_online_test_table_name('nycTaxiCITable')
 
     backfill_time = BackfillTime(start=datetime(
         2020, 5, 20), end=datetime(2020, 5, 20), step=timedelta(days=1))
@@ -165,7 +163,7 @@ def test_feature_swa_feature_gen_with_preprocessing():
     client.materialize_features(settings)
     # just assume the job is successful without validating the actual result in Redis. Might need to consolidate
     # this part with the test_feathr_online_store test case
-    client.wait_job_to_finish(timeout_sec=600)
+    client.wait_job_to_finish(timeout_sec=900)
 
     res = client.get_online_features(online_test_table, '265', ['f_location_avg_fare', 'f_location_max_fare'])
     assert res == [1000041.625, 1000100.0]
@@ -180,14 +178,14 @@ def test_feathr_get_offline_features_hdfs_source():
     client = basic_test_setup(os.path.join(test_workspace_dir, "feathr_config.yaml"))
 
     batch_source1 = HdfsSource(name="nycTaxiBatchSource_add_new_dropoff_and_fare_amount_column",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                               preprocessing=add_new_dropoff_and_fare_amount_column,
                               event_timestamp_column="new_lpep_dropoff_datetime",
                               # event_timestamp_column="lpep_dropoff_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
 
     batch_source2 = HdfsSource(name="nycTaxiBatchSource_add_new_fare_amount",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                               preprocessing=add_new_fare_amount,
                               event_timestamp_column="lpep_dropoff_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -248,7 +246,7 @@ def test_feathr_get_offline_features_hdfs_source():
     ]
 
     settings = ObservationSettings(
-        observation_path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+        observation_path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
         event_timestamp_column="lpep_dropoff_datetime",
         timestamp_format="yyyy-MM-dd HH:mm:ss")
 
@@ -281,7 +279,7 @@ def test_get_offline_feature_two_swa_with_diff_preprocessing():
     client = basic_test_setup(os.path.join(test_workspace_dir, "feathr_config.yaml"))
 
     swa_source_1 = HdfsSource(name="nycTaxiBatchSource1",
-                               path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                               path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                                preprocessing=add_new_dropoff_and_fare_amount_column,
                                event_timestamp_column="new_lpep_dropoff_datetime",
                                timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -313,7 +311,7 @@ def test_get_offline_feature_two_swa_with_diff_preprocessing():
 
 
     swa_source_2 = HdfsSource(name="nycTaxiBatchSource2",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
                               preprocessing=add_new_surcharge_amount_and_pickup_column,
                               event_timestamp_column="new_lpep_pickup_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -337,7 +335,7 @@ def test_get_offline_feature_two_swa_with_diff_preprocessing():
                                )
 
     swa_source_3 = HdfsSource(name="nycTaxiBatchSource3",
-                              path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04_old.csv",
+                              path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04_old.csv",
                               preprocessing=add_old_lpep_dropoff_datetime,
                               event_timestamp_column="old_lpep_dropoff_datetime",
                               timestamp_format="yyyy-MM-dd HH:mm:ss")
@@ -365,7 +363,7 @@ def test_get_offline_feature_two_swa_with_diff_preprocessing():
     ]
 
     settings = ObservationSettings(
-        observation_path="abfss://feathrazuretest3fs@feathrazuretest3storage.dfs.core.windows.net/demo_data/green_tripdata_2020-04.csv",
+        observation_path="wasbs://public@azurefeathrstorage.blob.core.windows.net/sample_data/green_tripdata_2020-04.csv",
         event_timestamp_column="lpep_dropoff_datetime",
         timestamp_format="yyyy-MM-dd HH:mm:ss")
 
